function video = SegmentVideoIntoSuperpixels(video, num_sup_from_user)

    if (~exist('num_sup_from_user', 'var') || isempty(num_sup_from_user))
        num_sup_from_user = 400;
    end
    
    num_frames = size(video.grayscale_frames, 3);
    video.superpixels = zeros(size(video.grayscale_frames));
    video.sup_idx_range = [];
    video.sup_centers = [];
    video.sup_area = [];
    video.sup_idx_range = [];
    video.supSpeed = [];
    video.supSpeedReliab = [];
    video.Vx = zeros(size(video.grayscale_frames));
    video.Vy = zeros(size(video.grayscale_frames));
    video.flow_reliab = false(size(video.grayscale_frames));
    
    % Sigma for weighting the flow of pixels in a superpixel
    sigma_motion = (video.width * video.height / num_sup_from_user);
    
    s_global = 1;
    use_pb = false;
    
    if (~use_pb)
        addpath(genpath('TurboPixels'));
    else
        addpath(genpath('TurboPixels_Pb'));
    end
    
    for i = 1:num_frames
        disp(['Processing frame ', num2str(i),' out of ', num2str(num_frames)]);
        img = video.grayscale_frames(:,:,i);
        
        if (i > 1)
            seeds = video.sup_centers(video.sup_idx_range(i-1,1):video.sup_idx_range(i-1,2), :);
            good_sups = repmat(video.supSpeedReliab(video.sup_idx_range(i-1,1):video.sup_idx_range(i-1,2)), [1,2]) > 0;
            flow = video.supSpeed(video.sup_idx_range(i-1,1):video.sup_idx_range(i-1,2), :);
            seeds(good_sups) = seeds(good_sups) + flow(good_sups);
            [M,N] = size(superPixels);
            seeds(:, 1) = max(min(round(seeds(:,1)), N), 1);
            seeds(:, 2) = max(min(round(seeds(:,2)), M), 1);
        else
            seeds = [];
        end

        if (~use_pb)
            [phi, boundary] = superpixels(img, num_sup_from_user, [], [], seeds);
        else
            pb = pbBGTG(img);
            lower_score = 0.02;
            higher_score = 0.05;
            thinned_score = hysthresh(pb, higher_score, lower_score);

            % Link the resulting branches into edges with a given minimum length
            min_edge_length = 10;
            edgelist = edgelink(thinned_score, min_edge_length);

            new_pb = zeros(size(pb));
            for e = 1:numel(edgelist)
                edge_idx = sub2ind(size(pb), edgelist{e}(:,1), edgelist{e}(:,2));
                mean_pb = mean(pb(edge_idx));
                new_pb(edge_idx) = mean_pb;
            end

            pb = new_pb;

            [phi, boundary] = superpixels_pb(img, num_sup_from_user, [], [], seeds, pb);
        end
        superPixels = bwlabel(~boundary, 4);
        num_sups = max(superPixels(:));
        stats = regionprops(superPixels, 'Centroid', 'Area');
        video.sup_idx_range(i, 1) = s_global;
        video.sup_idx_range(i, 2) = s_global + num_sups - 1;
        video.sup_centers = [video.sup_centers; reshape([stats(:).Centroid], [2, numel(stats)])'];
        video.sup_area = [video.sup_area; [stats(:).Area]'];
        video.superpixels(:,:,i) = superPixels;

        if (i < num_frames)
            [Vx,Vy,reliab] = optFlowLk(video.grayscale_frames(:,:,i),  video.grayscale_frames(:,:,i+1), [], 4, 3, 3e-6, 0 );
            reliab = reliab > 3e-6;
            video.Vx(:,:,i) = Vx;
            video.Vy(:,:,i) = Vy;
            video.flow_reliab(:,:,i) = reliab;

            for s = 1:num_sups
                sup_idx = video.superpixels(:,:,i) == s;
                reliable_pixels = reliab(sup_idx);
                numReliab = sum(reliable_pixels);

                [y,x] = find(sup_idx);
                weights = exp(-((x - video.sup_centers(s_global, 1)).^2 + ...
                    (y - video.sup_centers(s_global, 2)).^2) / sigma_motion);
                sum_weights = sum(weights(reliable_pixels));

                if (numReliab > 0)
                    video.supSpeed(s_global, 1) = sum(Vx(sup_idx & reliab).*weights(reliable_pixels)) / sum_weights;
                    video.supSpeed(s_global, 2) = sum(Vy(sup_idx & reliab).*weights(reliable_pixels)) / sum_weights;
                    video.supSpeedReliab(s_global,1) = (numReliab / video.sup_area(s_global)) > 0.1;
                else
                    video.supSpeed(s_global, 1) = 0;
                    video.supSpeed(s_global, 2) = 0;
                    video.supSpeedReliab(s_global,1) = false;
                end

                s_global = s_global + 1;
            end
        else
            video.supSpeed = [video.supSpeed; zeros(num_sups,2)];
            video.supSpeedReliab = [video.supSpeedReliab; zeros(num_sups,1)];
        end
    end

    video.supSpeedReliab = video.supSpeedReliab > 0;
